from zope.interface import Interface


class IFtwLogo(Interface):
    """ftw.logo Browser Layer
    """


class ILogo(Interface):
    """ Public interface for getting logo scales
    """

    def __init__(context, request):
        """Adapts context and request"""

    def get_config(config_type):
        """Get config based on config_type"""


class IIconConfig(Interface):
    """Adapter interface for logo customization adapter.
    """

    def __init__():
        """Adapts the zope application.
        """


class ILogoConfig(Interface):
    """Adapter interface for logo customization adapter.
    """

    def __init__():
        """Adapts the zope application.
        """



from ftw.logo.converter import convert
from ftw.logo.converter import make_transformer
from ftw.logo.converter import SCALES
from ftw.logo.image import Image
from ftw.logo.interfaces import IIconConfig
from ftw.logo.interfaces import ILogoConfig
from hashlib import sha256
from textwrap import wrap
from zope.configuration.exceptions import ConfigurationError
from zope.interface import implements


def get_cachekey_from_blob(*args):
    cachekey = sha256()
    for blob in args:
        if blob:
            for chunk in wrap(blob, 4096):
                cachekey.update(chunk)
    return cachekey.hexdigest()


class AbstractConfig(object):

    base = None
    logo = None
    mobile = None
    favicon = None
    primary_logo_scale = None

    def __init__(self, **kwargs):
        if 'base' not in kwargs:
            raise ConfigurationError('A base svg is required')

        self.base = Image(filename=kwargs['base'])

        if 'logo' in kwargs:
            self.logo = Image(filename=kwargs['logo'])
        if 'mobile' in kwargs:
            self.mobile = Image(filename=kwargs['mobile'])
        if 'favicon' in kwargs:
            self.favicon = Image(filename=kwargs['favicon'])

        if 'height' in kwargs:
            # Update by reference the transformer with the new height
            SCALES['LOGOS']['LOGO'] = make_transformer(height=kwargs['height'])
        if 'mobile_height' in kwargs:
            # Update by reference the transformer with the new height
            SCALES['LOGOS']['MOBILE_LOGO'] = make_transformer(height=kwargs['mobile_height'])

        self.cachekey = get_cachekey_from_blob(
            self.base.make_blob(),
            self.logo and self.logo.make_blob() or None,
            self.mobile and self.mobile.make_blob() or None,
            self.favicon and self.favicon.make_blob() or None,)
        self.scales = {}
        self.collect_scales()
        self.set_primary_logo_scale(**kwargs)

    def add_scale(self, name, scale):
        self.scales[name] = scale

    def collect_scales(self):
        raise NotImplemented()  # pragma: no cover

    def get_scale(self, name):
        return self.scales[name]

    def set_primary_logo_scale(self, **kwargs):
        if 'primary_logo_scale' in kwargs:
            self.primary_logo_scale = kwargs['primary_logo_scale']


class LogoConfig(AbstractConfig):
    """Logo config entry.
    """

    implements(ILogoConfig)

    def collect_scales(self):
        for scale in SCALES['LOGOS']:
            if getattr(self, scale.lower(), None):
                self.add_scale(scale, convert(getattr(self, scale.lower()), scale))
            else:
                self.add_scale(scale, convert(self.base, scale))


class IconConfig(AbstractConfig):
    """Icon config entry.
    """

    implements(IIconConfig)

    def collect_scales(self):
        for scale in SCALES['ICONS']:
            if getattr(self, scale.lower(), None):
                self.add_scale(scale, convert(getattr(self, scale.lower()), scale))
            else:
                self.add_scale(scale, convert(self.base, scale))


class AbstractConfigOverride(AbstractConfig):

    def __init__(self, blobImage):
        base_img = Image(blob=blobImage.data, format='svg')
        self.cachekey = get_cachekey_from_blob(blobImage.data)
        self.scales = {}
        self.collect_scales(base_img)


class LogoConfigOverride(AbstractConfigOverride):
    """Logo config (TTW) overrides.
    """

    implements(ILogoConfig)

    def collect_scales(self, base_img):
        for scale in SCALES['LOGOS']:
            self.add_scale(scale, convert(base_img, scale))


class IconConfigOverride(AbstractConfigOverride):
    """Icon config (TTW) overrides.
    """

    implements(IIconConfig)

    def collect_scales(self, base_img):
        for scale in SCALES['ICONS']:
            self.add_scale(scale, convert(base_img, scale))



from ftw.logo.interfaces import IIconConfig
from ftw.logo.interfaces import ILogo
from ftw.logo.interfaces import ILogoConfig
from ftw.logo.manual_override import BLOB_CACHEKEY
from ftw.logo.manual_override import OVERRIDES_FIXED_ID
from hashlib import sha256
from plone.app.caching.interfaces import IETagValue
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import Interface
from zope.interface import implementer


@implementer(ILogo)
@adapter(INavigationRoot, Interface)
class Logo(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_config(self, config_type):
        return getMultiAdapter((self.context, self.request), config_type)

    def get_logo_config(self):
        return getMultiAdapter((self.context, self.request), ILogoConfig)

    def get_icon_config(self):
        return getMultiAdapter((self.context, self.request), IIconConfig)


@implementer(IETagValue)
@adapter(Interface, Interface)
class LogoViewletETagValue(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        cachekey = sha256()
        cachekey.update(getMultiAdapter(
                (self.context, self.request), ILogo).get_logo_config().cachekey)
        cachekey.update(getMultiAdapter(
                (self.context, self.request), ILogo).get_icon_config().cachekey)

        # self.context is a navigation root
        overridesItem = self.context.get(OVERRIDES_FIXED_ID)
        if overridesItem is not None:
            annotations = IAnnotations(overridesItem)
            blob_cachekey = annotations.get(BLOB_CACHEKEY, '')
        else:
            blob_cachekey = ''
        cachekey.update(blob_cachekey)
        return cachekey.hexdigest()



from unittest import TestCase
from ftw.logo.logoconfig import LogoConfig
from ftw.logo.logoconfig import IconConfig
import os


source = os.path.join(os.path.dirname(__file__), 'fixtures/logo.svg')
img_source = os.path.join(os.path.dirname(__file__), 'fixtures/logo.png')


class TestCollecter(TestCase):

    def assertImage(self, img, width, height, format=None):
        self.assertEqual(img['width'], width)
        self.assertEqual(img['height'], height)
        if format:
            self.assertEqual(img['format'].lower(), format.lower())

    def test_all_logos_are_collected(self):
        component = LogoConfig(base=source)

        self.assertEqual(len(component.scales), 3, 'Should store three scales')
        self.assertImage(component.get_scale('BASE'), 1, 1, 'svg')
        self.assertImage(component.get_scale('LOGO'), 80, 80, 'png')
        self.assertImage(component.get_scale('MOBILE_LOGO'), 50, 50, 'png')

    def test_all_icons_are_collected(self):
        component = IconConfig(base=source)

        self.assertEqual(len(component.scales), 8, 'Should store eight scales')
        self.assertImage(component.get_scale('BASE'), 1, 1, 'svg')
        self.assertImage(component.get_scale('APPLE_TOUCH_ICON'), 180, 180, 'png')
        self.assertImage(component.get_scale('FAVICON_32X32'), 32, 32, 'png')
        self.assertImage(component.get_scale('FAVICON_16X16'), 16, 16, 'png')
        self.assertImage(component.get_scale('MSTILE_150X150'), 150, 150, 'png')
        self.assertImage(component.get_scale('ANDROID_192X192'), 192, 192, 'png')
        self.assertImage(component.get_scale('ANDROID_512X512'), 512, 512, 'png')
        self.assertImage(component.get_scale('FAVICON'), 16, 16, 'ico')

    def test_add_specific_logo_as_config(self):
        component = LogoConfig(base=source, logo=img_source)

        self.assertEqual(len(component.scales), 3, 'Should store one scales')
        self.assertImage(component.get_scale('LOGO'), 762, 80, 'png')

        self.assertImage(component.get_scale('BASE'), 1, 1, 'svg')
        self.assertImage(component.get_scale('MOBILE_LOGO'), 50, 50, 'png')

    def test_primary_logo_scale(self):
        component = LogoConfig(base=source, logo=img_source,
                               primary_logo_scale='logo')
        self.assertEquals('logo', component.primary_logo_scale)



from ftw.logo.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from ftw.logo.testing import get_etag_value_for


class TestLogoViewlet(FunctionalTestCase):

    @browsing
    def test_logo_tag(self, browser):
        etag = get_etag_value_for(self.portal, self.request)
        browser.login().visit(self.portal)

        self.assertEqual(
            map(lambda x: x.attrib['href'], browser.css('#portal-logo')),
            ['http://nohost/plone'],
        )

        self.assertEqual(
            map(lambda x: x.attrib['src'], browser.css('#portal-logo > img')),
            ['http://nohost/plone/@@logo/logo/get_logo?r={}'.format(etag)],
        )

    @browsing
    def test_mobile_logo_tag(self, browser):
        etag = get_etag_value_for(self.portal, self.request)
        browser.login().visit(self.portal)

        self.assertEqual(
            map(lambda x: x.attrib['href'], browser.css('#portal-mobilelogo')),
            ['http://nohost/plone'],
        )

        self.assertEqual(
            map(lambda x: x.attrib['src'], browser.css('#portal-mobilelogo > img')),
            ['http://nohost/plone/@@logo/logo/MOBILE_LOGO?r={}'.format(etag)],
        )



from ftw.logo.interfaces import IIconConfig
from ftw.logo.interfaces import ILogoConfig
from ftw.logo.logoconfig import IconConfig
from ftw.logo.logoconfig import LogoConfig
from zope import schema
from zope.component.zcml import handler
from zope.configuration import fields
from zope.configuration.fields import GlobalInterface
from zope.interface import Interface


class ILogoDirective(Interface):

    for_ = GlobalInterface(
        title=u'The interface the context should provide.',
        required=False)

    layer = GlobalInterface(
        title=u'The interface the request should provide.',
        required=False)

    base = fields.Path(
        title=u'Relative path to the logo svg file.',
        required=True)

    logo = fields.Path(
        title=u'Relative path to the logo image file.',
        required=False)

    mobile = fields.Path(
        title=u'Relative path to the mobile logo image file.',
        required=False)

    primary_logo_scale = schema.Text(
        title=u'Relative path to the mobile logo image file.',
        required=False)

    height = schema.Text(
        title=u'Relative path to the mobile logo image file.',
        required=False)

    mobile_height = schema.Text(
        title=u'Relative path to the mobile logo image file.',
        required=False)


class IIconDirective(Interface):

    for_ = GlobalInterface(
        title=u'The interface the context should provide.',
        required=False)

    layer = GlobalInterface(
        title=u'The interface the request should provide.',
        required=False)

    base = fields.Path(
        title=u'Relative path to the icon file.',
        required=True)

    favicon = fields.Path(
        title=u'Relative path to the favicon file.',
        required=False)


def registerLogo(_context, **kwargs):
    component = LogoConfig(**kwargs)

    def adapter_factory(context, request):
        return component

    requires = (kwargs.get('for_', Interface),
                kwargs.get('layer', Interface))

    _context.action(
        discriminator=('logo:logo',) + requires,
        callable=handler,
        args=('registerAdapter', adapter_factory, requires, ILogoConfig, ''))


def registerIcon(_context, **kwargs):
    component = IconConfig(**kwargs)

    def adapter_factory(context, request):
        return component

    requires = (kwargs.get('for_', Interface),
                kwargs.get('layer', Interface))

    _context.action(
        discriminator=('logo:icon',) + requires,
        callable=handler,
        args=('registerAdapter', adapter_factory, requires, IIconConfig, ''))



from copy import deepcopy
from ftw.logo import converter
from ftw.logo.testing import get_etag_value_for
from ftw.logo.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from wand.exceptions import CorruptImageError
from wand.image import Image
from zope.interface import Interface
import os


source_path = os.path.join(os.path.dirname(__file__), 'fixtures')
custom = os.path.join(source_path, 'custom.svg')
png = os.path.join(source_path, 'green.png')
logo = os.path.join(source_path, 'logo.png')


class IDummyLayer(Interface):
    pass


class TestLogoView(FunctionalTestCase):

    def setUp(self):
        super(TestLogoView, self).setUp()

        # Isolate changes on SCALES
        self.origin_scales = deepcopy(converter.SCALES)

    def tearDown(self):
        super(TestLogoView, self).tearDown()
        converter.SCALES = self.origin_scales

    def verify_image_format(self, browser, view, expected_format):
        browser.visit(self.portal, view=view)
        self.assertEqual(200, browser.status_code)
        try:
            im = Image(blob=browser.contents, format=expected_format)
        except CorruptImageError:    # pragma: no cover
            self.fail("Image is incorrect format - expected {}".format(expected_format))
        return im

    @browsing
    def test_logo_view(self, browser):
        self.verify_image_format(browser, '@@logo/logo/BASE', 'svg')

    @browsing
    def test_icon_view(self, browser):
        self.verify_image_format(browser, '@@logo/icon/BASE', 'svg')

    @browsing
    def test_logo_scale(self, browser):
        im = self.verify_image_format(browser, '@@logo/logo/MOBILE_LOGO', 'png')
        self.assertEqual(50, im.height)

    @browsing
    def test_icon_scale(self, browser):
        im = self.verify_image_format(browser, '@@logo/icon/ANDROID_192X192', 'png')
        self.assertEqual(192, im.height)
        self.assertEqual(192, im.width)

    @browsing
    def test_not_found(self, browser):
        with browser.expect_http_error(code=404):
            browser.login().visit(self.portal, view='@@logo/logo/not_found')

        with browser.expect_http_error(code=404):
            browser.login().visit(self.portal, view='@@logo/not_found')

    def test_etag_value_invalidates(self):
        before = get_etag_value_for(self.portal, self.request)
        self.layer['load_zcml_string']('''
        <configure
            xmlns:logo="https://namespaces.4teamwork.ch/ftw.logo"
            i18n_domain="my.package"
            package="ftw.logo.tests">
            <logo:logo base="{}" />
        </configure>
        '''.format(custom))
        after = get_etag_value_for(self.portal, self.request)

        self.assertNotEqual(
            before,
            after)

    @browsing
    def test_special_get_logo_scale_name_returns_svg_by_default(self, browser):
        self.verify_image_format(browser, '@@logo/logo/get_logo', 'svg')
        self.grant('Site Administrator')
        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')

        with open(png) as green_png:
            browser.fill({'Standard (desktop) logo (PNG)': green_png}).submit()
            self.verify_image_format(browser, '@@logo/logo/get_logo', 'png')

    @browsing
    def test_special_get_logo_scale_name_returns_uploaded_png(self, browser):
        self.verify_image_format(browser, '@@logo/logo/get_logo', 'svg')
        self.grant('Site Administrator')
        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')

        with open(png) as green_png:
            browser.fill({'Standard (desktop) logo (PNG)': green_png}).submit()
            self.verify_image_format(browser, '@@logo/logo/get_logo', 'png')

    @browsing
    def test_special_get_logo_scale_name_returns_uploaded_svg(self, browser):
        self.verify_image_format(browser, '@@logo/logo/get_logo', 'svg')
        self.grant('Site Administrator')
        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')

        with open(png) as green_png:
            browser.fill({'Standard (desktop) logo (PNG)': green_png}).submit()
            self.verify_image_format(browser, '@@logo/logo/get_logo', 'png')

    def test_primary_logo_scale(self):
        self.layer['load_zcml_string']('''
        <configure
            xmlns:logo="https://namespaces.4teamwork.ch/ftw.logo"
            i18n_domain="my.package"
            package="ftw.logo.tests">
            <logo:logo base="{}" primary_logo_scale="logo" />
        </configure>
        '''.format(custom))

        # Simulate publishTraverse of '@@logo/logo/get_logo'
        view = self.portal.restrictedTraverse('@@logo')
        view.publishTraverse(view.request, u"logo")
        view.publishTraverse(view.request, u"get_logo")
        view()
        self.assertEquals('LOGO', view.scale)

    def test_change_logo_height(self):
        self.layer['load_zcml_string']('''
        <configure
            xmlns:logo="https://namespaces.4teamwork.ch/ftw.logo"
            i18n_domain="my.package"
            package="ftw.logo.tests">
            <logo:logo base="{}" height="100" mobile_height="30" />
        </configure>
        '''.format(custom))

        # Simulate publishTraverse of '@@logo/logo/get_logo'
        view = self.portal.restrictedTraverse('@@logo')
        view.publishTraverse(view.request, u"logo")
        view.publishTraverse(view.request, u"LOGO")
        img = Image(blob=view())
        self.assertEquals(100, img.width)

        # Simulate publishTraverse of '@@logo/logo/get_logo'
        view = self.portal.restrictedTraverse('@@logo')
        view.publishTraverse(view.request, u"logo")
        view.publishTraverse(view.request, u"MOBILE_LOGO")
        img = Image(blob=view())
        self.assertEquals(30, img.width)



from ftw.logo import _
from ftw.logo.logoconfig import get_cachekey_from_blob
from ftw.logo.logoconfig import IconConfigOverride
from ftw.logo.logoconfig import LogoConfigOverride
from hashlib import sha256
from plone.dexterity.browser.view import DefaultView
from plone.dexterity.browser import edit
from plone.dexterity.utils import createContentInContainer
from plone.dexterity.utils import iterSchemata
from plone.namedfile.field import INamedBlobImage
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.interface import Invalid
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.schema import getFieldsInOrder
import transaction


OVERRIDES_FIXED_ID = 'ftw-logo-overrides'
# Annotations keys
LOGO_OVERRIDES_KEY = 'ftw.logo.logo_overrides'
ICON_OVERRIDES_KEY = 'ftw.logo.icon_overrides'
BLOB_CACHEKEY = 'ftw.logo.blob_cachekey'
OVERRIDES_KEY_PATTERN = 'ftw.logo.{}_overrides'


def svg_file_only(value):
    if value.contentType != 'image/svg+xml':
        raise Invalid(
            u"This image must be a SVG file " +
            u"({} supplied)".format(value.contentType)
        )
    return True


def png_file_only(value):
    if value.contentType != 'image/png':
        raise Invalid(
            u"This image must be a PNG file " +
            u"({} supplied)".format(value.contentType)
        )
    return True


class IManualOverrides(model.Schema):

    # Note: The field names are carefully constructed as one of:
    #   logo_<logo scale name>, or
    #   icon_<icon scale name>
    # The form (manual_override.pt) depends on this to find the ZCML
    # default image

    logo_BASE = NamedBlobImage(
        title=_(u"SVG base logo"),
        description=_(u"Overriding the base logo will generate an override "
                      u"for BOTH logos below if not already set."),
        required=False,
        constraint=svg_file_only,
    )

    logo_LOGO = NamedBlobImage(
        title=_(u"Standard (desktop) logo (PNG)"),
        required=False,
        constraint=png_file_only,
    )

    logo_MOBILE_LOGO = NamedBlobImage(
        title=_(u"Mobile logo (PNG)"),
        required=False,
        constraint=png_file_only,
    )

    icon_BASE = NamedBlobImage(
        title=_(u"SVG base icon"),
        description=_(u"Overriding the base icon will generate an "
                      u"override for ALL icons below if not already set."),
        required=False,
        constraint=svg_file_only,
    )

    icon_APPLE_TOUCH_ICON = NamedBlobImage(
        title=_(u"Apple touch icon"),
        required=False,
        constraint=png_file_only,
    )

    icon_FAVICON_32X32 = NamedBlobImage(
        title=_(u"Favicon 32x32"),
        required=False,
        constraint=png_file_only,
    )

    icon_FAVICON_16X16 = NamedBlobImage(
        title=_(u"Favicon 16x16"),
        required=False,
        constraint=png_file_only,
    )

    icon_MSTILE_150X150 = NamedBlobImage(
        title=_(u"Mstile icon 150x150"),
        required=False,
        constraint=png_file_only,
    )

    icon_ANDROID_192X192 = NamedBlobImage(
        title=_(u"Android icon 192x192"),
        required=False,
        constraint=png_file_only,
    )

    icon_ANDROID_512X512 = NamedBlobImage(
        title=_(u"Android icon 512x512"),
        required=False,
        constraint=png_file_only,
    )

    icon_FAVICON = NamedBlobImage(
        title=_(u"Favicon"),
        required=False,
        constraint=png_file_only,
    )


@adapter(IManualOverrides, IObjectModifiedEvent)
def overrides_changed(override_object, event):
    annotations = IAnnotations(override_object)
    if override_object.logo_BASE:
        annotations[LOGO_OVERRIDES_KEY] = LogoConfigOverride(override_object.logo_BASE)
    if override_object.icon_BASE:
        annotations[ICON_OVERRIDES_KEY] = IconConfigOverride(override_object.icon_BASE)

    # calculate a cachekey combining all blob fields in this object
    cachekey = sha256()
    for schemata in iterSchemata(override_object):
        for name, field in getFieldsInOrder(schemata):
            try:
                value = field.get(schemata(override_object))
                if value is not field.missing_value and field.schema == INamedBlobImage:
                    blobdata = field.get(override_object).data
                    cachekey.update("|{}".format(get_cachekey_from_blob(blobdata)))
                    continue
            except AttributeError:
                pass
            cachekey.update("|")
    annotations[BLOB_CACHEKEY] = cachekey.hexdigest()


class CreateOverridesIfReqdForm(BrowserView):
    """
    Create IManualOverrides if it does not exist and redirect to it's edit form
    """
    def __call__(self):
        navroot = self.context
        overridesItem = navroot.get(OVERRIDES_FIXED_ID)
        if overridesItem is None:

            overridesItem = createContentInContainer(
                navroot,
                'ftw.logo.ManualOverrides',
                checkConstraints=False,
                id=OVERRIDES_FIXED_ID,
                title='Logo and Icon Overrides',
                description='Manual overrides for the site logo(s) and icons',
            )
            transaction.get().commit()

            self.request.response.redirect('{}/@@edit'.format(
                overridesItem.absolute_url_path()
            ))
            return
        self.request.response.redirect('{}/view'.format(
            overridesItem.absolute_url_path()
        ))
        return


class ManualOverrideMixin(object):
    def get_origin_for_scale(self, fullscalename):
        """ Lookup HTML class for depending on authoritative origin for a particular scale """
        fieldname = fullscalename.replace('/', '_')
        config_name = fullscalename.split('/')[0]
        if getattr(self.context, fieldname):
            return 'direct_override'
        if getattr(self.context, '{}_BASE'.format(config_name)):
            return 'scaled_base_override'
        else:
            return 'zcml'


class EditManualOverrideForm(edit.DefaultEditForm, ManualOverrideMixin):
    label = _(u"Edit Manual Logo and Icon Overrides")
    description = _(u"Site logos and icons set in ZCML are shown in " +
                    u"the left column. Overrides from this form are " +
                    u"shown in the right column")

    template = ViewPageTemplateFile('manual_override.pt')

    def get_origin_for_scale(self, fullscalename):
        """ Lookup HTML class for depending on authoritative origin for a particular scale """
        fieldname = fullscalename.replace('/', '_')
        config_name = fullscalename.split('/')[0]
        if getattr(self.context, fieldname):
            return 'direct_override'
        if getattr(self.context, '{}_BASE'.format(config_name)):
            return 'scaled_base_override'
        else:
            return 'zcml'


class ManualOverrideView(DefaultView, ManualOverrideMixin):
    pass



from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter


BROWSERCONFIG_TEMPLATE = '''
<?xml version="1.0" encoding="utf-8"?>
<browserconfig>
<msapplication>
<tile>
<square150x150logo src="{}/@@logo/icon/MSTILE_150X150"/>
<TileColor>#da532c</TileColor>
</tile>
</msapplication>
</browserconfig>
'''


class BrowserconfigView(BrowserView):

    def __call__(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        navigation_root_url = portal_state.navigation_root_url()
        response = self.request.response
        response.setHeader('Content-Type', 'text/xml')

        return BROWSERCONFIG_TEMPLATE.replace('\n', '').format(navigation_root_url)



from plone.app.layout.viewlets.common import ViewletBase
from zope.component import getMultiAdapter
from plone.app.caching.interfaces import IETagValue


class LogoBaseViewlet(ViewletBase):

    def get_etag_value(self):
        navigation_root = self.portal_state.navigation_root()
        adapter = getMultiAdapter((navigation_root, self.request),
                                  IETagValue,
                                  name='logo-viewlet')
        return adapter()

    def update(self):
        super(LogoBaseViewlet, self).update()
        self.cachekey = self.get_etag_value()
        self.portal_state = getMultiAdapter((self.context, self.request),
                                            name=u'plone_portal_state')
        self.navigation_root_url = self.portal_state.navigation_root_url()



from ftw.logo.manual_override import BLOB_CACHEKEY
from ftw.logo.manual_override import ICON_OVERRIDES_KEY
from ftw.logo.manual_override import LOGO_OVERRIDES_KEY
from Products.CMFCore.utils import getToolByName
from StringIO import StringIO
from zope.annotation.interfaces import IAnnotations


def uninstall(self):
    clean_up_content_annotations(self)

    setup_tool = getToolByName(self, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile(
        'profile-ftw.logo:uninstall',
        ignore_dependencies=True)


def clean_up_content_annotations(portal):
    """
    Remove objects from content annotations in Plone site,

    This is mostly to remove objects which might make the site un-exportable
    when eggs / Python code have been removed.
    """
    output = StringIO()

    portal_catalog = getToolByName(portal, 'portal_catalog')
    brains = portal_catalog(object_provides='ftw.logo.manual_override.IManualOverrides')

    for b in brains:
        override_obj = b.getObject()
        annotations = IAnnotations(override_obj)
        for key in (ICON_OVERRIDES_KEY, LOGO_OVERRIDES_KEY, BLOB_CACHEKEY):
            if key in annotations:
                del annotations[key]
                print >> output, "Cleaned up annotation {} on {}".format(key, b.getPath())

    return output



from ftw.upgrade import UpgradeStep


class InstallFtwTheming(UpgradeStep):
    """install ftw.theming.
    """

    def __call__(self):
        self.install_upgrade_profile()
        self.ensure_profile_installed('profile-ftw.theming:default')



from ftw.logo.interfaces import IIconConfig
from ftw.logo.interfaces import ILogoConfig
from ftw.logo.testing import META_ZCML
from unittest import TestCase
from zope.component import getMultiAdapter
from zope.configuration.xmlconfig import ZopeXMLConfigurationError
from zope.interface import implementer
from zope.interface import Interface
from zope.interface.verify import verifyObject
import os


source_path = os.path.join(os.path.dirname(__file__), 'fixtures')
logo = os.path.join(source_path, 'logo.svg')
icon = os.path.join(source_path, 'logo.svg')
custom = os.path.join(source_path, 'custom.svg')

logo_img = os.path.join(source_path, 'logo.png')
mobile_img = os.path.join(source_path, 'mobile.png')


class IDummyLayer(Interface):
    pass


@implementer(IDummyLayer)
class DummyRequest(object):
    pass


class TestZCML(TestCase):
    layer = META_ZCML

    def test_logo_component(self):
        self.load_zcml('<logo:logo base="{}" />'.format(logo))
        registry = getMultiAdapter((None, None), ILogoConfig)
        verifyObject(ILogoConfig, registry)

    def test_logo_component_with_other_image_attr(self):
        self.load_zcml('<logo:logo base="{}" logo="{}" />'.format(logo, logo_img))
        registry = getMultiAdapter((None, None), ILogoConfig)
        verifyObject(ILogoConfig, registry)

        self.load_zcml('<logo:logo base="{}" mobile="{}" />'.format(logo, mobile_img))
        registry = getMultiAdapter((None, None), ILogoConfig)
        verifyObject(ILogoConfig, registry)

    def test_logo_component_is_overwriteable(self):
        self.load_zcml(
            '<logo:logo base="{}" />'.format(logo),
            '<logo:logo base="{}"'.format(custom),
            '  layer="ftw.logo.tests.test_zcml.IDummyLayer" />')
        registry = getMultiAdapter((object(), DummyRequest()), ILogoConfig)
        verifyObject(ILogoConfig, registry)

    def test_icon_component(self):
        self.load_zcml(
            '<logo:icon base="{}" />'.format(icon))
        registry = getMultiAdapter((None, None), IIconConfig)
        verifyObject(IIconConfig, registry)

    def test_icon_component_is_overwriteable(self):
        self.load_zcml(
            '<logo:icon base="{}" />'.format(icon),
            '<logo:icon base="{}"'.format(custom),
            '  layer="ftw.logo.tests.test_zcml.IDummyLayer" />')
        registry = getMultiAdapter((object(), DummyRequest()), IIconConfig)
        verifyObject(IIconConfig, registry)

    def load_zcml(self, *lines):
        self.layer.load_zcml_string('\n'.join((
            '<configure ',
            '    xmlns:logo="https://namespaces.4teamwork.ch/ftw.logo"',
            '    i18n_domain="my.package"',
            '    package="ftw.logo.tests">',
        ) + lines + (
            '</configure>',
        )))

    def test_fail_to_load_if_no_valid_attr_is_present(self):
        with self.assertRaises(ZopeXMLConfigurationError):
            self.load_zcml('<logo:logo />'.format(logo))

        with self.assertRaises(ZopeXMLConfigurationError):
            self.load_zcml('<logo:logo dummy="nothing" />'.format(logo))

        with self.assertRaises(ZopeXMLConfigurationError):
            self.load_zcml('<logo:icon />'.format(logo))

        with self.assertRaises(ZopeXMLConfigurationError):
            self.load_zcml('<logo:icon dummy="nothing" />'.format(logo))



from ftw.logo.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestBrowserconfigView(FunctionalTestCase):

    @browsing
    def test_browserconfig_view(self, browser):
        browser.login().visit(self.portal, view='browserconfig.xml')
        self.assertEqual('text/xml', browser.mimetype)
        self.assertMultiLineEqual(
            '<?xml version="1.0" encoding="utf-8"?>'
            '<browserconfig>'
            '<msapplication>'
            '<tile>'
            '<square150x150logo src="http://nohost/plone/@@logo/icon/MSTILE_150X150"/>'
            '<TileColor>#da532c</TileColor>'
            '</tile>'
            '</msapplication>'
            '</browserconfig>', browser.contents)



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.testing import IS_PLONE_5
from ftw.testing.layer import COMPONENT_REGISTRY_ISOLATION
from ftw.testing.layer import ComponentRegistryLayer
from plone.app.caching.interfaces import IETagValue
from plone.app.testing import FunctionalTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.testing import z2
from zope.component import getMultiAdapter
from zope.configuration import xmlconfig
import os


source_path = os.path.join(os.path.dirname(__file__), 'tests', 'fixtures')
blue_svg = os.path.join(source_path, 'blue.svg')


class MetaZCMLLayer(ComponentRegistryLayer):

    def setUp(self):
        super(MetaZCMLLayer, self).setUp()
        import ftw.logo
        self.load_zcml_file('meta.zcml', ftw.logo)


META_ZCML = MetaZCMLLayer()


class LogoLayer(PloneSandboxLayer):
    defaultBases = (COMPONENT_REGISTRY_ISOLATION, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.logo')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.logo:default')


LOGO_FIXTURE = LogoLayer()
LOGO_FUNCTIONAL = FunctionalTesting(
    bases=(LOGO_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.logo:functional")


class BlueBaseLogoLayer(PloneSandboxLayer):
    defaultBases = (COMPONENT_REGISTRY_ISOLATION, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            ('<configure xmlns="http://namespaces.zope.org/zope"'
             '    xmlns:logo="https://namespaces.4teamwork.ch/ftw.logo">'
             '  <include package="z3c.autoinclude" file="meta.zcml" />'
             '  <includePlugins package="plone" />'
             '  <includePluginsOverrides package="plone" />'
             '  <logo:logo base="{0}"/>'
             '  <logo:icon base="{0}"/>'
             '</configure>').format(blue_svg),
            context=configurationContext)

        z2.installProduct(app, 'ftw.logo')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.logo:default')

        if IS_PLONE_5:
            applyProfile(portal, 'plone.app.contenttypes:default')


BLUE_BASE_LOGO_FIXTURE = BlueBaseLogoLayer()
BLUE_BASE_LOGO_FUNCTIONAL = FunctionalTesting(
    bases=(BLUE_BASE_LOGO_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.logo:bluebasefunctional")


def get_etag_value_for(context, request):
    adapter = getMultiAdapter((context, request),
                              IETagValue,
                              name='logo-viewlet')
    return adapter()



from ftw.logo.image import Image


def make_raw():
    def raw(img):
        return img
    return raw


def make_resizer(width=0, height=0, extension='png'):
    def resizer(img):
        img = img.clone()
        img.sample(width, height)
        return Image(image=img.convert(extension))
    return resizer


def make_transformer(width='', height='', extension='png'):
    def transformer(img):
        img = img.clone()
        img.transform(resize='{}x{}'.format(width, height))
        return Image(image=img.convert(extension))
    return transformer


def make_ico_converter():
    ICO_SCALES = [
        make_resizer(16, 16),
        make_resizer(32, 32),
        make_resizer(48, 48),
    ]

    def converter(src):
        scales = map(lambda resizer: resizer(src), ICO_SCALES)

        def merge_scales(first, other):
            first.format = 'ico'
            first.append(other)
            other.close()
            return first

        return reduce(merge_scales, scales)
    return converter


def flatten_scales(scales):
    merged = scales['LOGOS'].copy()
    merged.update(scales['ICONS'])
    return merged


SCALES = {
    'LOGOS': {
        'LOGO':             make_transformer(height=80),
        'MOBILE_LOGO':      make_transformer(height=50),
        'BASE':             make_raw(),
    },
    'ICONS': {
        'APPLE_TOUCH_ICON': make_resizer(180, 180),
        'FAVICON_32X32':    make_resizer(32, 32),
        'FAVICON_16X16':    make_resizer(16, 16),
        'MSTILE_150X150':   make_resizer(150, 150),
        'ANDROID_192X192':  make_resizer(192, 192),
        'ANDROID_512X512':  make_resizer(512, 512),
        'FAVICON':          make_ico_converter(),
        'BASE':             make_raw(),
    }
}


def convert(source, scale):
    scales = flatten_scales(SCALES)
    if scale not in scales:
        raise Exception('scale: {} is not supported'.format(scale))
    scale = scales[scale](source)
    result = {'extension': scale.extension,
              'width': scale.width,
              'height': scale.height,
              'format': scale.format,
              'sequence_length': len(scale.sequence)}
    result['data'] = scale.make_blob()  # closes scale as side effect
    return result



from Products.CMFCore.utils import getToolByName
from ftw.builder import Builder
from ftw.builder import create
from ftw.logo.manual_override import OVERRIDES_FIXED_ID
from ftw.logo.testing import BLUE_BASE_LOGO_FUNCTIONAL
from ftw.logo.testing import get_etag_value_for
from ftw.logo.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from wand.color import Color
from wand.exceptions import CorruptImageError
from wand.image import Image
from zope.annotation.interfaces import IAnnotations
import os


source_path = os.path.join(os.path.dirname(__file__), 'fixtures')
blue_svg = os.path.join(source_path, 'blue.svg')
red_svg = os.path.join(source_path, 'red.svg')
green_png = os.path.join(source_path, 'green.png')


def get_rgb(color):
    # Helper Function to get rgb and rgba for a color
    color_dict = {'red': 'rgb{}(255, 0, 0, 1)',
                  'green': 'rgb{}(0, 255, 0, 1)',
                  'blue': 'rgb{}(0, 0, 255, 1)'}

    return [color_dict[color].format(''), color_dict[color].format('a')]


class TestManualOverrides(FunctionalTestCase):
    layer = BLUE_BASE_LOGO_FUNCTIONAL

    @browsing
    def test_permission_for_overrides_form(self, browser):

        # test editor user can't use add form
        self.grant('Editor')
        browser.login()
        with browser.expect_unauthorized():
            browser.visit(self.portal, view='@@logo-and-icon-overrides')

        self.assertFalse(browser.visit(self.portal).css('#personaltools-override_logos'))

        # test site admin CAN use the overrides form
        self.grant('Site Administrator')
        browser.visit(self.portal, view='@@logo-and-icon-overrides')
        self.assertEqual(200, browser.status_code)

        self.assertTrue(browser.visit(self.portal).css('#personaltools-override_logos'))

        # test editor user can't view the content type
        self.grant('Editor')
        with browser.expect_unauthorized():
            browser.login().visit(self.portal[OVERRIDES_FIXED_ID])

        # test editor user can't view the content type
        self.grant('Site Administrator')
        browser.login().visit(self.portal[OVERRIDES_FIXED_ID])
        self.assertEqual(200, browser.status_code)

    @browsing
    def test_addable_only_on_navroots(self, browser):
        self.grant('Site Administrator')
        browser.login()
        folder = create(Builder('folder'))
        with browser.expect_http_error(code=404):
            browser.login().visit(folder, view='@@logo-and-icon-overrides')

        # If we make the folder a navroot then we should be able to access the form on it
        # FIXME - 500 from ComponentLookupError in LogoViewletETagValue
        # alsoProvides(self.portal['folder'], INavigationRoot)
        # transaction.commit()
        # browser.visit(folder, view='@@logo-and-icon-overrides')
        # self.assertEqual(200, browser.status_code)

    def verify_correct_image(self, browser, view, expected_format, expected_color=None):
        """
        Verify that a logo/icon at a given view is as expected

        expected_format ('png', 'svg' etc.)
        expected_color - proves the *source* of the image by looking at the histogram
            - we use ZCML to set the base SVG image to 100% 'blue'
            - the TTW SVG (if set) is 100% 'red'
            - the TTW PNG (if set) is 100% GREEN ('#0f0')
        """
        # We logout to test images are *publicly* visible
        browser.logout()
        browser.visit(self.portal, view=view)
        self.assertEqual(200, browser.status_code)
        try:
            im = Image(blob=browser.contents, format=expected_format)
        except CorruptImageError:    # pragma: no cover
            self.fail("Image is incorrect format - expected {}".format(expected_format))
        if (expected_color):
            # Determine the source (SVG, PNG) by using the colour
            # Test that 90% of image is expected_color
            total_pixels = im.height * im.width
            im.alpha_channel = 'opaque'
            try:
                # I try rgb value as key otherwise rgba.
                value = (im.histogram.get(Color(get_rgb(expected_color)[0])) or
                         im.histogram.get(Color(get_rgb(expected_color)[1])))
                self.assertGreater(value, total_pixels * 0.9)
            except KeyError:    # pragma: no cover
                self.fail(("expected_color {} not found in image histogram\n"
                           "First 5 colours were: {}").format(Color(expected_color), im.histogram.keys()[:5]))
        return im

    @browsing
    def test_logo_overrides(self, browser):
        # We use primary colours to check the source of the scaled image

        self.grant('Site Administrator')
        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')
        self.assertEqual(200, browser.status_code)
        with open(red_svg) as svg_file:
            browser.fill({'SVG base logo': svg_file}).submit()

        # Check SVG image is set
        self.verify_correct_image(browser, '@@logo/logo/BASE', 'svg',
                                  'red')
        # Check conversion to PNG
        img = self.verify_correct_image(browser, '@@logo/logo/MOBILE_LOGO',
                                        'png', 'red')
        self.assertEqual(50, img.height)
        # Check ZCML 'bypass' - base SVG should be shown
        img = self.verify_correct_image(browser, '@@logo/z/logo/MOBILE_LOGO',
                                        'png', 'blue')
        self.assertEqual(50, img.height)

        # verify icons are unaffected (i.e. still derive from base icon)
        self.verify_correct_image(browser, '@@logo/icon/BASE', 'svg',
                                  'blue')
        self.verify_correct_image(browser, '@@logo/icon/ANDROID_192X192',
                                  'png', 'blue')
        # Test PNG override
        # (Note: we don't test dimensions as they are not enforced)
        overrides = self.portal[OVERRIDES_FIXED_ID]
        browser.login().visit(overrides, view='@@edit')
        with open(green_png) as png_file:
            browser.fill({'Mobile logo (PNG)': png_file}).submit()

        self.verify_correct_image(browser, '@@logo/logo/MOBILE_LOGO', 'png',
                                  'green')

        # Check ZCML 'bypass' - a scale of the base SVG should be shown
        self.verify_correct_image(browser, '@@logo/z/logo/MOBILE_LOGO', 'png',
                                  'blue')

        # verify icons are unaffected (i.e. still derive from base icon)
        self.verify_correct_image(browser, '@@logo/icon/BASE', 'svg',
                                  'blue')
        self.verify_correct_image(browser, '@@logo/icon/ANDROID_192X192',
                                  'png', 'blue')

    @browsing
    def test_icon_overrides(self, browser):
        # We use primary colours to check the source of the scaled image

        self.grant('Site Administrator')
        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')
        self.assertEqual(200, browser.status_code)
        with open(red_svg) as svg_file:
            browser.fill({'SVG base icon': svg_file}).submit()

        # Check SVG image is set
        self.verify_correct_image(browser, '@@logo/icon/BASE', 'svg',
                                  'red')
        # Check conversion to PNG
        img = self.verify_correct_image(browser, '@@logo/icon/FAVICON_32X32',
                                        'png', 'red')
        self.assertEqual(32, img.height)
        self.assertEqual(32, img.width)
        # Check ZCML 'bypass' - a scale of the base SVG should be shown
        img = self.verify_correct_image(browser, '@@logo/z/icon/FAVICON_32X32',
                                        'png', 'blue')
        self.assertEqual(32, img.height)
        self.assertEqual(32, img.width)

        # verify logos are unaffected (i.e. still derive from base logo)
        self.verify_correct_image(browser, '@@logo/logo/BASE', 'svg',
                                  'blue')
        self.verify_correct_image(browser, '@@logo/logo/LOGO', 'png',
                                  'blue')

        # Test PNG override (Note: we don't test dimensions as they are not enforced)
        overrides = self.portal[OVERRIDES_FIXED_ID]
        browser.login().visit(overrides, view='@@edit')
        with open(green_png) as png_file:
            browser.fill({'Favicon 32x32': png_file}).submit()
        self.verify_correct_image(browser, '@@logo/icon/FAVICON_32X32', 'png',
                                  'green')

        # Check ZCML 'bypass' - base SVG should be shown
        self.verify_correct_image(browser, '@@logo/z/icon/FAVICON_32X32',
                                  'png', 'blue')

        # verify logos are unaffected (i.e. still derive from base logo)
        self.verify_correct_image(browser, '@@logo/logo/BASE', 'svg',
                                  'blue')
        self.verify_correct_image(browser, '@@logo/logo/LOGO', 'png',
                                  'blue')

    @browsing
    def test_form_validation(self, browser):
        self.grant('Site Administrator')

        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')
        with open(green_png) as png_file:
            browser.fill({'SVG base logo': png_file}).submit()
        self.assertIn('This image must be a SVG file (image/png supplied)', browser.contents)

        overrides = self.portal[OVERRIDES_FIXED_ID]
        browser.login().visit(overrides, view='@@edit')
        with open(red_svg) as svg_file:
            browser.fill({'Apple touch icon': svg_file}).submit()
        self.assertIn('This image must be a PNG file (image/svg+xml supplied)', browser.contents)

    @browsing
    def test_uninstall_removes_annotations_removed(self, browser):

        self.grant('Site Administrator')

        # add some overrides
        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')
        with open(red_svg) as svg_file:
            browser.fill({'SVG base logo': svg_file,
                          'SVG base icon': svg_file}).submit()

        # Check we have some annotations
        override_obj = self.portal[OVERRIDES_FIXED_ID]
        annotations = IAnnotations(override_obj)
        self.assertGreater(len(annotations), 0, 'Sanity check annotations added')

        # Uninstall
        quick_installer_tool = getToolByName(self.layer['portal'],
                                             'portal_quickinstaller')
        quick_installer_tool.uninstallProducts(['ftw.logo'])

        # check all annotations are gone
        annotations = IAnnotations(override_obj)
        self.assertEqual(len(annotations), 0, 'Uninstallation should remove all annotations')

    def assertDistinctETags(self, etagList, extra_msg=''):
        if len(etagList) != len(set(etagList)):
            self.fail("ETag values are not distinct {} {}".format(etagList, extra_msg))

    @browsing
    def test_etag_for_overrides(self, browser):
        """
        Test that the etag changes for each change of configuration, so caching works
        """
        self.grant('Site Administrator')
        etagValues = []
        etagValues.append(get_etag_value_for(self.portal, self.request))
        browser.login().visit(self.portal, view='@@logo-and-icon-overrides')
        with open(red_svg) as svg_file:
            browser.fill({'SVG base icon': svg_file}).submit()

        etagValues.append(get_etag_value_for(self.portal, self.request))
        self.assertDistinctETags(etagValues, 'after overriding base SVG icon')

        overrides = self.portal[OVERRIDES_FIXED_ID]
        browser.login().visit(overrides, view='@@edit')
        with open(green_png) as png_file:
            browser.fill({'Favicon 32x32': png_file}).submit()

        etagValues.append(get_etag_value_for(self.portal, self.request))
        self.assertDistinctETags(etagValues, 'after overriding PNG icon')

        browser.login().visit(overrides, view='@@edit')
        with open(green_png) as png_file:
            browser.fill({'Android icon 512x512': png_file}).submit()

        etagValues.append(get_etag_value_for(self.portal, self.request))
        self.assertDistinctETags(etagValues, 'after overriding a second PNG icon')



from ZPublisher.Iterators import IStreamIterator
from zope.interface import implementer
from StringIO import StringIO


@implementer(IStreamIterator)
class StringIOStreamIterator(StringIO):

    def __len__(self):
        return self.len



from ftw.logo.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestWebmanifestView(FunctionalTestCase):

    @browsing
    def test_webmanifest_view(self, browser):
        browser.login().visit(self.portal, view='manifest.json')
        self.assertEqual('application/json', browser.mimetype)
        self.assertEqual({
            "theme_color": "#ffffff",
            "name": "logo",
            "short_name": "logo",
            "icons": [
                {
                    "src": "http://nohost/plone/@@logo/icon/ANDROID_192X192",
                    "type": "image/png",
                    "sizes": "192x192"
                },
                {
                    "src": "http://nohost/plone/@@logo/icon/ANDROID_512X512",
                    "type": "image/png",
                    "sizes": "512x512"
                }
            ],
            "background_color": "#ffffff",
            "display": "standalone"
        }, browser.json)



from ftw.logo.converter import convert
from ftw.logo.image import Image
from unittest import TestCase
import os


source = os.path.join(os.path.dirname(__file__), 'fixtures/logo.svg')
base = Image(filename=source)


class TestConverter(TestCase):

    def assertImage(self, img, width, height, format=None, blob=None):
        self.assertEqual(img['width'], width)
        self.assertEqual(img['height'], height)
        if format:
            self.assertEqual(img['format'].lower(), format.lower())
        if blob:
            self.assertEqual(img['data'], blob)

    def test_rejects_unsupported_types(self):
        with self.assertRaises(Exception) as context:
            convert(base, 'UNKNOWN')

        self.assertTrue('scale: UNKNOWN is not supported' in context.exception)

    def test_converts_svg_base(self):
        self.assertImage(
            convert(base, 'BASE'), 1, 1, 'svg',
            '<?xml version="1.0" encoding="UTF-8"?>\n'
            '<svg width="1px" height="1px" viewBox="0 0 1 1" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">\n'
            '    <g id="logo"></g>\n'
            '</svg>\n'
        )

    def test_converts_png_fixed_size(self):
        self.assertImage(
            convert(base, 'APPLE_TOUCH_ICON'), 180, 180, 'png')

    def test_converts_png_fixed_aspect_ratio(self):
        self.assertImage(
            convert(base, 'LOGO'), 80, 80, 'png')

    def test_converts_multipart_images(self):
        img = convert(base, 'FAVICON')
        self.assertEqual(img['sequence_length'], 3,
                         'Should contain three subimages in the sequence')



from ftw.logo.converter import SCALES
from ftw.logo.converter import flatten_scales
from ftw.logo.interfaces import IIconConfig
from ftw.logo.interfaces import ILogo
from ftw.logo.interfaces import ILogoConfig
from ftw.logo.manual_override import OVERRIDES_FIXED_ID
from ftw.logo.manual_override import OVERRIDES_KEY_PATTERN
from ftw.logo.StringIOStreamIterator import StringIOStreamIterator
from plone.app.layout.globals.interfaces import IViewView
from plone.namedfile.browser import DisplayFile
from Products.Five.browser import BrowserView
from zExceptions import BadRequest
from zExceptions import NotFound
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse
import mimetypes


CONFIGS = {
    'logo': ILogoConfig,
    'icon': IIconConfig,
}


@implementer(IPublishTraverse, IViewView)
class LogoView(BrowserView):

    def __init__(self, context, request):
        super(LogoView, self).__init__(context, request)
        self.scale = None
        self.config = None
        self.zcml_only = None

    def publishTraverse(self, request, name):
        if self.zcml_only is None and self.config is None and name == 'z':
            self.zcml_only = True
            return self
        elif self.config is None and name in CONFIGS:
            self.config_name = name
            self.config = CONFIGS[name]
            return self
        elif self.config and name in flatten_scales(SCALES):
            self.scale = name
            return self
        elif self.config and name == 'get_logo':
            self.scale = name
            return self
        else:
            raise NotFound()

    def __call__(self):
        if not self.config or not self.scale:
            raise BadRequest()

        if self.scale == 'get_logo':
            return self.handle_get_logo()

        check_overrides = not self.zcml_only
        return ((check_overrides and self.get_dx_overridden_image()) or
                self.get_zcml_configured_image())

    def get_dx_overridden_image(self):
        overridesItem = self.context.get(OVERRIDES_FIXED_ID)
        if overridesItem is None:
            return None
        # check if requested scale has been overridden and return it
        field_name = '{}_{}'.format(self.config_name, self.scale)
        field = getattr(overridesItem, field_name)
        if not field:
            # check if base logo/icon  has been overridden, then return the transformed BASE logo/icon
            base_field_name = '{}_BASE'.format(self.config_name)
            if getattr(overridesItem, base_field_name):
                annotations = IAnnotations(overridesItem)
                config = annotations.get(OVERRIDES_KEY_PATTERN.format(self.config_name), None)
                if config:
                    return self.show_config_scale(config)
        if not field:
            return None

        display_file = DisplayFile(overridesItem, self.request).publishTraverse(self.request, field_name or base_field_name)
        return display_file()

    def get_zcml_configured_image(self):
        config = getMultiAdapter(
            (self.context, self.request), ILogo).get_config(self.config)
        return self.show_config_scale(config)

    def show_config_scale(self, config):
        scale = config.get_scale(self.scale)
        response = self.request.response
        iterator = StringIOStreamIterator(scale['data'])
        extension = scale.get('extension') or scale['format'].lower()
        contenttype = mimetypes.types_map.get('.{}'.format(extension),
                                              'application/octet-stream')
        response.setHeader('X-Theme-Disabled', 'True')
        charset = '' if contenttype == 'application/octet-stream' else 'charset=utf-8'
        response.setHeader(
            'Content-Type', '{}; {}'.format(contenttype, charset))
        response.setHeader('Content-Length', iterator.len)
        if self.request.get('r'):
            # Do not set cache headers when no cachekey provided.
            # The cached representation is to be considered fresh for 1 year
            # http://stackoverflow.com/a/3001556/880628
            response.setHeader('Cache-Control', 'public, max-age=31536000')
        return iterator

    def handle_get_logo(self):
        if self.has_dx_logo() and not self.has_dx_base():
            self.scale = 'LOGO'
            return self.get_dx_overridden_image()
        elif self.has_dx_base():
            self.scale = 'BASE'
            return self.get_dx_overridden_image()
        else:
            self.scale = 'BASE'

            config = getMultiAdapter(
                (self.context, self.request), ILogo).get_config(self.config)
            if config.primary_logo_scale:
                self.scale = config.primary_logo_scale.upper()

            return self.get_zcml_configured_image()

    def has_dx_logo(self):
        overridesItem = self.context.get(OVERRIDES_FIXED_ID)
        if not overridesItem:
            return False
        return bool(overridesItem.logo_LOGO)

    def has_dx_base(self):
        overridesItem = self.context.get(OVERRIDES_FIXED_ID)
        if not overridesItem:
            return False
        return bool(overridesItem.logo_BASE)



from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
import json


class WebmanifestView(BrowserView):

    def __call__(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        navigation_root_url = portal_state.navigation_root_url()
        response = self.request.response
        response.setHeader('Content-Type', 'application/json')

        return json.dumps({
            'name': 'logo',
            'short_name': 'logo',
            'icons': [
                {
                    'src': '{}/@@logo/icon/ANDROID_192X192'
                    .format(navigation_root_url),
                    'sizes': '192x192',
                    'type': 'image/png'
                },
                {
                    'src': '{}/@@logo/icon/ANDROID_512X512'
                    .format(navigation_root_url),
                    'sizes': '512x512',
                    'type': 'image/png'
                }
            ],
            'theme_color': '#ffffff',
            'background_color': '#ffffff',
            'display': 'standalone'
        })



from wand.image import Image
import os


class Image(Image):

    def __init__(self, **kwargs):
        self.filename = kwargs.get("filename", None)
        # behind format is a setter so the extension has to be kept seperately
        self.extension = kwargs.pop("extension", None)
        if self.filename and not os.path.isfile(self.filename):
            raise ValueError('{} could not be found'.format(self.filename))  # pragma: no cover

        super(Image, self).__init__(**kwargs)
        if self.filename:
            if self.extension is None:
                _, extension = os.path.splitext(self.filename)
                self.extension = extension.lstrip('.')

            self.format = self.extension

        self._blob = None

    def make_blob(self):
        if self._blob:
            return self._blob
        if self.extension == 'svg' and self.filename:
            # If the target extension is svg, and we assume that the input data is also svg,
            # we don't need to transform anything, but we can simply return the original data
            # read from self.filename.
            # self contains the image, but it is already converted to MVG, but we cannot
            # convert it back to SVG. Therefore we need to read the original.
            with open(self.filename, 'r') as f:
                self._blob = f.read()
        else:
            self._blob = super(Image, self).make_blob(self.extension)
        return self._blob

    def append(self, other):
        self.sequence.append(other)



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.logo.browser.logo_base_viewlet import LogoBaseViewlet


class LogoViewlet(LogoBaseViewlet):

    index = ViewPageTemplateFile('logo_viewlet.pt')



from ftw.logo.testing import get_etag_value_for
from ftw.logo.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from ftw.testing import IS_PLONE_5
from unittest import skip
from unittest import skipIf


class TestIconViewlet(FunctionalTestCase):

    @skip("TODO: Remove defualt viewport from sunburst default template")
    @browsing
    def test_viewport_metatag(self, browser):
        browser.login().visit(self.portal)

        self.assertEqual(
            ['width=device-width, initial-scale=1'],
            map(lambda x: x.attrib['content'], browser.css(
                'meta[name="viewport"]'))
        )

    @skipIf(IS_PLONE_5, 'The apple touch logo is not configurable in plone5')
    @browsing
    def test_logo_viewlet_displays_relevant_metadata_in_header_part_one(self, browser):
        etag = get_etag_value_for(self.portal, self.request)
        browser.login().visit(self.portal)

        self.assertEqual(
            [{
                'href': 'http://nohost/plone/@@logo/icon/APPLE_TOUCH_ICON?r={}'.format(etag),
                'sizes': '180x180'}],
            map(lambda x: {
                'href': x.attrib['href'],
                'sizes': x.attrib['sizes'],
            }, browser.css('link[rel="apple-touch-icon"]'))
        )

    @browsing
    def test_logo_viewlet_displays_relevant_metadata_in_header_part_two(self, browser):
        etag = get_etag_value_for(self.portal, self.request)
        browser.login().visit(self.portal)

        self.assertEqual(
            [
                {
                    'type': 'image/png',
                    'sizes': '32x32',
                    'href': 'http://nohost/plone/@@logo/icon/FAVICON_32X32?r={}'.format(etag),
                },
                {
                    'type': 'image/png',
                    'sizes': '16x16',
                    'href': 'http://nohost/plone/@@logo/icon/FAVICON_16X16?r={}'.format(etag),
                },
                {
                    'type': '',
                    'sizes': '',
                    'href': 'http://nohost/plone/@@logo/icon/FAVICON?r={}'.format(etag),
                },
            ],
            map(lambda x: {
                'type': x.attrib.get('type', ''),
                'sizes': x.attrib.get('sizes', ''),
                'href': x.attrib['href'],
            }, browser.css('link[rel="icon"]'))
        )

        self.assertEqual(
            ['http://nohost/plone/manifest.json?r={}'.format(etag)],
            map(lambda x: x.attrib['href'], browser.css(
                'link[rel="manifest"]'))
        )

        self.assertEqual(
            ['http://nohost/plone/browserconfig.xml?r={}'.format(etag)],
            map(lambda x: x.attrib['content'], browser.css(
                'meta[name="msapplication-config"]'))
        )

        self.assertEqual(
            ['#ffffff'],
            map(lambda x: x.attrib['content'], browser.css(
                'meta[name="theme-color"]'))
        )
